@@ -27,7 +27,9 @@ def _get_attr_args_and_kwargs_from_ax_call_item(ax_call_item):
     return attr, args, kwargs
 
 
-def multi_row_plot(data_list=(), plot_func=plt.plot, figsize=3, plot_func_kwargs=None, ax_calls=()):
+def multi_row_plot(
+    data_list=(), plot_func=plt.plot, figsize=3, plot_func_kwargs=None, ax_calls=()
+):
     """
     Quickly plotting multiple rows of data.
 
@@ -74,13 +76,17 @@ def multi_row_plot(data_list=(), plot_func=plt.plot, figsize=3, plot_func_kwargs
             plot_func(row_data, **plot_func_kwargs)
 
         ax = plt.gca()
-        for attr, args, kwargs in map(_get_attr_args_and_kwargs_from_ax_call_item, specific_ax_calls):
+        for attr, args, kwargs in map(
+            _get_attr_args_and_kwargs_from_ax_call_item, specific_ax_calls
+        ):
             getattr(ax, attr)(*args, **kwargs)
 
         ax_list.append(ax)
 
     for ax in ax_list:
-        for attr, args, kwargs in map(_get_attr_args_and_kwargs_from_ax_call_item, ax_calls):
+        for attr, args, kwargs in map(
+            _get_attr_args_and_kwargs_from_ax_call_item, ax_calls
+        ):
             getattr(ax, attr)(*args, **kwargs)
 
     return ax_list
@@ -89,18 +95,20 @@ def multi_row_plot(data_list=(), plot_func=plt.plot, figsize=3, plot_func_kwargs
 import numpy as np
 
 
-def ax_func_to_plot(list_func_per_ax,
-                    n_per_row=3,
-                    title=None,
-                    title_font_size=10,
-                    width=15,
-                    height_row=10,
-                    saving_path=None,
-                    rec_padding=(0, 0, 0, 0),
-                    x_labels=None,
-                    y_labels=None,
-                    outer_axis_labels_only=False,
-                    show=True):
+def ax_func_to_plot(
+    list_func_per_ax,
+    n_per_row=3,
+    title=None,
+    title_font_size=10,
+    width=15,
+    height_row=10,
+    saving_path=None,
+    rec_padding=(0, 0, 0, 0),
+    x_labels=None,
+    y_labels=None,
+    outer_axis_labels_only=False,
+    show=True,
+):
     """
     Each function in list_func_per_ax takes an ax as input and draw something on it
 
@@ -109,7 +117,9 @@ def ax_func_to_plot(list_func_per_ax,
     y_labels: the label on all the y-axis
     """
     n_rows = int(np.ceil(len(list_func_per_ax) / n_per_row))
-    fig, axes = plt.subplots(nrows=n_rows, ncols=n_per_row, figsize=(width, height_row * n_rows))
+    fig, axes = plt.subplots(
+        nrows=n_rows, ncols=n_per_row, figsize=(width, height_row * n_rows)
+    )
     for ax, func in zip(axes.flatten(), list_func_per_ax):
         func(ax)
     # fig.legend(fancybox=True, framealpha=1, shadow=True, borderpad=1)
@@ -132,9 +142,9 @@ def ax_func_to_plot(list_func_per_ax,
 
 
 if __name__ == '__main__':
+
     def ax_func(ax):
         ax.plot([1, 5, 3])
         ax.set_title('test_test')
 
-
     ax_func_to_plot([ax_func] * 6, title='Test', x_labels='x_name_here')
